package eu.jonahbauer.raytracing.render;

import org.jetbrains.annotations.NotNull;

public record Color(double r, double g, double b) {
    public static final @NotNull Color BLACK = new Color(0.0, 0.0, 0.0);
    public static final @NotNull Color WHITE = new Color(1.0, 1.0, 1.0);
    public static final @NotNull Color SKY = new Color(0.5, 0.7, 1.0);
    public static final @NotNull Color RED = new Color(1.0, 0.0, 0.0);
    public static final @NotNull Color GREEN = new Color(0.0, 1.0, 0.0);
    public static final @NotNull Color BLUE = new Color(0.0, 0.0, 1.0);

    public static @NotNull Color lerp(@NotNull Color a, @NotNull Color b, double t) {
        if (t < 0) return a;
        if (t > 1) return b;
        return new Color(
                (1 - t) * a.r + t * b.r,
                (1 - t) * a.g + t * b.g,
                (1 - t) * a.b + t * b.b
        );
    }

    public static @NotNull Color multiply(@NotNull Color a, @NotNull Color b) {
        return new Color(a.r() * b.r(), a.g() * b.g(), a.b() * b.b());
    }

    public Color {
        if (r < 0 || r > 1 || g < 0 || g > 1 || b < 0 || b > 1) {
            throw new IllegalArgumentException("r, g and b must be in the range 0 to 1");
        }
    }
    
    public Color(int red, int green, int blue) {
        this(red / 255f, green / 255f, blue / 255f);
    }

    public int red() {
        return (int) (255.99 * r);
    }

    public int green() {
        return (int) (255.99 * g);
    }

    public int blue() {
        return (int) (255.99 * b);
    }
}
